package de.ids_mannheim.korap.openthesaurus;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;

public class ProcessJSON {

    static ObjectMapper mapper = new ObjectMapper().enable(SerializationFeature.INDENT_OUTPUT);

    public static ObjectNode createKoralGroupNodeOfSynonyms(List<List<String>> synsets, ObjectNode initialtokenNode) {
        ObjectNode synonymsKoralGroupNode = mapper.createObjectNode();
        ArrayNode synonymsArrayNode = mapper.createArrayNode();

        synonymsKoralGroupNode.put("@type", "koral:group");
        synonymsKoralGroupNode.put("operation", "operation:sequence");
        for (List<String> synset : synsets) {
            for (String synonym : synset) {
                ObjectNode synonymNode = initialtokenNode.deepCopy();
                ((ObjectNode) synonymNode.get("wrap")).put("key", synonym);
                synonymsArrayNode.add(synonymNode);
            }
        }
        synonymsKoralGroupNode.set("operands", synonymsArrayNode);
        return synonymsKoralGroupNode;
    }

    public static ObjectNode traverseQueryNode(ObjectNode node) {

        String koralSpanType = node.get("@type").asText();

        if (koralSpanType.equals("koral:group")) {
            ObjectNode outputNode = mapper.createObjectNode();
            Iterator<Map.Entry<String, JsonNode>> koralGroupNodeIterator = node.fields();
            while (koralGroupNodeIterator.hasNext()) {
                Map.Entry<String, JsonNode> fieldEntry = koralGroupNodeIterator.next();
                String fieldEntryName = fieldEntry.getKey();
                if ((fieldEntryName.equals("operands"))) {
                    ArrayNode operandsModified = mapper.createArrayNode();
                    ArrayNode operands = (ArrayNode) fieldEntry.getValue();
                    for (int i = 0; i < operands.size(); i++) {
                        ObjectNode operand = (ObjectNode) operands.get(i);
                        operandsModified.add(traverseQueryNode(operand));
                    }
                    outputNode.set("operands", operandsModified);
                }
                else outputNode.set(fieldEntryName, fieldEntry.getValue());
            }
            return outputNode;
        }

        // when koralSpanType.equals("koral:token"). Are there any other cases?
        else {
            List<List<String>> synsets = DoOnStartup.thesaurusDict.get(node.at("/wrap/key").asText());
            return createKoralGroupNodeOfSynonyms(synsets, node);
        }
    }

    public static String processJSONFromString(String InputJSON) {

        try {
            Iterator<Map.Entry<String, JsonNode>> inputJSONTree = mapper.readTree(InputJSON).fields();
            ObjectNode outputJSONTree = mapper.createObjectNode();

            while (inputJSONTree.hasNext()) {
                Map.Entry<String, JsonNode> entry = inputJSONTree.next();
                if (Objects.equals(entry.getKey(), "query")) {
                    ObjectNode processed = traverseQueryNode((ObjectNode) entry.getValue());
                    outputJSONTree.set("query", processed);
                }
                else outputJSONTree.set(entry.getKey(), entry.getValue());
            }
            return mapper.writeValueAsString(outputJSONTree);
        }
        catch (JsonProcessingException e) {
            throw new RuntimeException("The given query is not a valid JSON!");
            //return InputJSON;
        }
    }

    public static void main(String[] args) throws IOException {
        File inputFile  = new File("src/main/resources/test_query.json");
        File outputFile = new File("src/main/resources/test_query_output.json");

        String jsonInput = Files.readString(Path.of("src/main/resources/test_query.json"));
        BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));

        writer.write(processJSONFromString(reader.));
        writer.close();
    }
}
